"{ Package: 'stx:libjava' }"

AbstractJavaTestCase subclass:#JavaInterfaceMethodRefTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Java-Tests-RuntimeConstantPool'
!


!JavaInterfaceMethodRefTests methodsFor:'javaInterfaceMethodRef tests'!

testCorrectInstanceCreation
    | initString  javaMethodRef |

    initString := 'Ljava/lang/Runnable;'.
    javaMethodRef := self 
                getInterfaceMethodRefNamed: 'run'
                typed: '()V'
                inClass: initString.
    self assertTrue: (javaMethodRef isResolved not).
    self assertTrue: (javaMethodRef valueCache isNil).
    self assertTrue: (javaMethodRef name = 'run').
    self assertTrue: (javaMethodRef descriptor = '()V').
    self assertTrue: (javaMethodRef classRef name = 'Ljava/lang/Runnable;').

    "Created: / 08-04-2011 / 14:01:41 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 23-05-2011 / 17:19:00 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testCorrectResolving
    | initString  javaMethodRef  expectedResult  result |

    initString := 'Ljava/lang/Runnable;'.
    javaMethodRef := self 
                getInterfaceMethodRefNamed: 'run'
                typed: '()V'
                inClass: initString.
    result := javaMethodRef resolve.
    expectedResult := (Java classForName: 'java.lang.Runnable') 
                methodDictionary at: #'run()V'.
    self assertTrue: (result = expectedResult).

    "Created: / 08-04-2011 / 14:07:57 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 23-05-2011 / 17:18:54 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidation
    | javaMethodRef  initString|

   initString :='Ljava/lang/Runnable;'.

    javaMethodRef := javaMethodRef := self 
                getInterfaceMethodRefNamed: 'run'
                typed: '()V'
                inClass: initString.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaMethodRef classRef isResolved).
    javaMethodRef invalidate.
    self assertTrue: (javaMethodRef isResolved not).
    self assertTrue: (javaMethodRef classRef isResolved not).

    "Created: / 08-04-2011 / 14:09:06 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 23-05-2011 / 17:19:45 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidationForClassNegative
    | javaMethodRef  initString|

    initString :='Ljava/lang/Runnable;'.

    javaMethodRef := javaMethodRef := self 
                getInterfaceMethodRefNamed: 'run'
                typed: '()V'
                inClass: initString.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaMethodRef classRef isResolved).
    javaMethodRef invalidateForClass: 'Ljava/lang/Object;'.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaMethodRef classRef isResolved).

    "Created: / 08-04-2011 / 16:23:06 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 23-05-2011 / 17:20:18 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidationForClassPositive
    | javaMethodRef  initString|

    initString :='Ljava/lang/Runnable;'.

    javaMethodRef := javaMethodRef := self 
                getInterfaceMethodRefNamed: 'run'
                typed: '()V'
                inClass: initString.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaMethodRef classRef isResolved).
    javaMethodRef invalidateForClass: 'Ljava/lang/Runnable;'.
    self assertTrue: (javaMethodRef isResolved not).
    self assertTrue: (javaMethodRef classRef isResolved not).

    "Created: / 08-04-2011 / 16:23:19 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 23-05-2011 / 17:20:51 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testResolving
    | javaMethodRef  initString|

     initString :='Ljava/lang/Runnable;'.

    javaMethodRef := javaMethodRef := self 
                getInterfaceMethodRefNamed: 'run'
                typed: '()V'
                inClass: initString.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaMethodRef classRef isResolved).
    self assertTrue: (javaMethodRef isResolved).

    "Created: / 08-04-2011 / 14:04:01 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 23-05-2011 / 17:21:10 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaInterfaceMethodRefTests methodsFor:'permission tests'!

testAccessingPublicFromSubclass
    | javaMethodRef  initString |

    initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    self 
        should: 
            [ javaMethodRef := self 
                        getInterfaceMethodRefNamed: 'publicMethod'
                        typed: '()Ljava/lang/String;'
                        inClass: initString.
            javaMethodRef resolve. ]
        raise: Error
        suchThat: [:e | e messageText = 'IncompatibleClassChangeError' ].

    "Created: / 13-04-2011 / 14:49:11 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 23-05-2011 / 17:54:12 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaInterfaceMethodRefTests class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
